#!/usr/bin/php -q
<?php
# $Id: loghandler.php,v 1.1 2004/10/18 12:20:14 exon Exp $
#
# Author: Andreas Ericsson <ae@op5.se>
#
# Copyright(C) 2004 OP5 AB
# All rights reserved.
#

# use signal handling
declare(ticks = 1);

###############################################################################
#
# Functions
#
#

# write to logs for later perusal
function append_logline($ip, $type, $msg) {
	$fname = '/var/www/html/logs/' . $ip . '_' . $type . '.log';
	if(($fd = fopen($fname, 'a'))) {
		fwrite($fd, $msg);
		fclose($fd);
		return(true);
	}
	return(false);
}

# handle a log message
function handle_logmsg($line) {
	$str = preg_split('/ +/', $line, 5);
	$when = sprintf("%s %s %s", $str[0], $str[1], $str[2]);
	$host = $str[3];
	$msg = "$when $str[4]";

	# resolve the hostname and add to proper log-file.
	$ip = gethostbyname($host);

	# XXX possibly change $line to $msg here
	append_logline($ip, 'logmsg', $line);
}


###############################################################################
#
# Main thread
#
#

# some global variables (very scarce)
$msg_fifo = '/opt/monitor/op5/loghandler/log_pipe';
$pid_dir = '/var/run';


# pre-fork
# create pid-file as root, with safe permissions
umask(0077);
$pid = pcntl_fork();
if($pid === -1)
	die("Failed to become daemon\n");
if($pid === 0)
	exit(0);

posix_setsid();

$pid = pcntl_fork();
if($pid === -1)
	die("Failed to become daemon\n");

if($pid) {
	$fd = fopen($pid_dir . '/loghandler.pid', 'w');
	fwrite($fd, $pid . "\n");
	fclose($fd);
	echo "loghandler daemon running with pid $pid\n";
	exit(0);
}

# drop privileges if we're being run as root
$run_user = 'apache';
$run_group = 'apache';
if(posix_geteuid() === 0) {
	$run_uid = posix_getpwnam($run_user);
	$run_gid = posix_getgrnam($run_group);
	posix_setgid($run_gid['gid']);
	posix_setuid($run_uid['uid']);
	posix_seteuid($run_uid['uid']);
}

# ignore SIGCHLD. This will prevent child processes from
# becoming zombies, and we really don't care about their
# return values anyway, since we can't do anything reasonable
# if one of them fail
pcntl_signal(SIGCHLD, SIG_IGN);

# replace daemons
fclose(STDIN);
fclose(STDOUT);
fclose(STDERR);
$stdin = fopen('/dev/null', 'r'); // set fd/0
$stdout = fopen('/dev/null', 'w'); // set fd/1
$stderr = fopen('php://stdout', 'w'); // hack to duplicate fd/1 to 2

# set a proper umask and create the fifo
umask(0002);
posix_mkfifo($msg_fifo, 0660);

while(true) {
	$fd = fopen($msg_fifo, 'r');
	if(!$fd)
		die("Failed to open $msg_fifo for reading\n");

	while(($line = fgets($fd))) {
		# not exactly fail-safe, but it scales better than not forking at all
		# and prevents that we miss anything in case we get stuck on
		# gethostbyname() (which shouldn't happen).
		if(!pcntl_fork()) {
			# child does some stuff
			handle_logmsg($line);
			exit(0);
		}
	}

	fclose($fd);
}

?>
